@@ -29,14 +29,15 @@
 #include "qgstemporalrangeobject.h"
 #include "qgsmapcanvasinteractionblocker.h"
 #include "qgsproject.h"
+#include "qgsdistancearea.h"
 
 #include <QDomDocument>
 #include <QGraphicsView>
-#include <QtCore>
 
 #include "qgsmapsettings.h" // TEMPORARY
 #include "qgsprevieweffect.h" //for QgsPreviewEffect::PreviewMode
 
+#include <QTimer>
 #include <QGestureEvent>
 #include "qgis_gui.h"
 
@@ -70,6 +71,7 @@ class QgsSnappingUtils;
 class QgsRubberBand;
 class QgsMapCanvasAnnotationItem;
 class QgsReferencedRectangle;
+class QgsRenderedItemResults;
 
 class QgsTemporalController;
 
@@ -82,7 +84,7 @@ class QgsMapMouseEvent;
  * \brief Map canvas is a class for displaying all GIS data types on a canvas.
  */
 
-class GUI_EXPORT QgsMapCanvas : public QGraphicsView
+class GUI_EXPORT QgsMapCanvas : public QGraphicsView, public QgsExpressionContextGenerator
 {
 
 #ifdef SIP_RUN
@@ -133,9 +135,9 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     const QgsMapSettings &mapSettings() const SIP_KEEPREFERENCE;
 
     /**
-     * Sets the temporal controller, tQgsMapCanvasInteractionBlockerhis controller will be used to
-     * update the canvas temporal range.
+     * Sets the temporal \a controller for this canvas.
      *
+     * The controller will be used to update the canvas' temporal range.
      * \since QGIS 3.14
      */
     void setTemporalController( QgsTemporalController *controller );
@@ -149,7 +151,7 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     const QgsTemporalController *temporalController() const;
 
     /**
-     * sets destination coordinate reference system
+     * Sets destination coordinate reference system
      * \since QGIS 2.4
      */
     void setDestinationCrs( const QgsCoordinateReferenceSystem &crs );
@@ -158,13 +160,28 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      * Resets the \a flags for the canvas' map settings.
      * \since QGIS 3.0
      */
-    void setMapSettingsFlags( QgsMapSettings::Flags flags );
+    void setMapSettingsFlags( Qgis::MapSettingsFlags flags );
 
     /**
-     * Gets access to the labeling results (may be NULLPTR)
+     * Gets access to the labeling results (may be NULLPTR).
+     *
+     * Since QGIS 3.20, if the \a allowOutdatedResults flag is FALSE then outdated labeling results (e.g.
+     * as a result of an ongoing canvas render) will not be returned, and instead NULLPTR will be returned.
+     *
      * \since QGIS 2.4
      */
-    const QgsLabelingResults *labelingResults() const;
+    const QgsLabelingResults *labelingResults( bool allowOutdatedResults = true ) const;
+
+    /**
+     * Gets access to the rendered item results (may be NULLPTR), which includes the results of rendering
+     * annotation items in the canvas map.
+     *
+     * If the \a allowOutdatedResults flag is FALSE then outdated rendered item results (e.g.
+     * as a result of an ongoing canvas render) will not be returned, and instead NULLPTR will be returned.
+     *
+     * \since QGIS 3.22
+     */
+    const QgsRenderedItemResults *renderedItemResults( bool allowOutdatedResults = true ) const;
 
     /**
      * Set whether to cache images of rendered layers
@@ -184,6 +201,12 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      */
     void clearCache();
 
+    /**
+     * Cancel any rendering job, in a blocking way. Used for application closing.
+     * \note not available in Python bindings
+     */
+    void cancelJobs() SIP_SKIP;
+
     /**
      * Blocks until the rendering job has finished.
      *
@@ -230,9 +253,28 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
 
     //! Returns the current zoom extent of the map canvas
     QgsRectangle extent() const;
-    //! Returns the combined extent for all layers on the map canvas
+
+    /**
+     * Returns the combined extent for all layers on the map canvas.
+     *
+     * This method returns the combined extent for all layers which are currently visible in the map canvas.
+     * The returned extent will be in the same CRS as the map canvas.
+     *
+     * \see projectExtent()
+     */
     QgsRectangle fullExtent() const;
 
+    /**
+     * Returns the associated project's full extent, in the canvas' CRS.
+     *
+     * This method returns the full extent for the project associated with this canvas.
+     * Unlike fullExtent(), this method does NOT consider which layers are actually visible in the map canvas.
+     *
+     * \see fullExtent()
+     * \since QGIS 3.20
+     */
+    QgsRectangle projectExtent() const;
+
     /**
      * Sets the extent of the map canvas to the specified rectangle.
      *
@@ -280,9 +322,23 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      */
     QgsPointXY center() const;
 
-    //! Zoom to the full extent of all layers
+    /**
+     * Zoom to the full extent of all layers currently visible in the canvas.
+     *
+     * \see zoomToProjectExtent()
+     */
     void zoomToFullExtent();
 
+    /**
+     * Zoom to the full extent the project associated with this canvas.
+     *
+     * This method zooms to the full extent for the project associated with this canvas.
+     * Unlike zoomToFullExtent(), this method does NOT consider which layers are actually visible in the map canvas.
+     *
+     * \since QGIS 3.20
+     */
+    void zoomToProjectExtent();
+
     //! Zoom to the previous extent (view)
     void zoomToPreviousExtent();
 
@@ -403,6 +459,18 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     //! Returns the map layer at position index in the layer stack
     QgsMapLayer *layer( int index );
 
+    /**
+     * Returns the map layer with the matching ID, or NULLPTR if no layers could be found.
+     *
+     * This method searches both layers associated with the map canvas (see layers())
+     * and layers from the QgsProject associated with the canvas
+     * (which is current the QgsProject::instance()). It can be used to resolve layer IDs to
+     * layers which may be visible in the canvas, but not associated with a QgsProject.
+     *
+     * \since QGIS 3.22
+     */
+    QgsMapLayer *layer( const QString &id );
+
     //! Returns number of layers on the map
     int layerCount() const;
 
@@ -545,7 +613,7 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     void enableAntiAliasing( bool flag );
 
     //! TRUE if antialiasing is enabled
-    bool antiAliasingEnabled() const { return mSettings.testFlag( QgsMapSettings::Antialiasing ); }
+    bool antiAliasingEnabled() const;
 
     //! sets map tile rendering flag
     void enableMapTileRendering( bool flag );
@@ -666,7 +734,9 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
      * \see setExpressionContextScope()
      * \since QGIS 3.4
      */
-    QgsExpressionContextScope *defaultExpressionContextScope() SIP_FACTORY;
+    QgsExpressionContextScope *defaultExpressionContextScope() const SIP_FACTORY;
+
+    QgsExpressionContext createExpressionContext() const override;
 
     /**
      * Sets the segmentation tolerance applied when rendering curved geometries
@@ -942,6 +1012,8 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     //! Renames the active map theme called \a theme to \a newTheme
     void mapThemeRenamed( const QString &theme, const QString &newTheme );
 
+    void updateDevicePixelFromScreen();
+
   signals:
 
     /**
@@ -953,6 +1025,15 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     //! Emitted when the scale of the map changes
     void scaleChanged( double );
 
+    /**
+     * Emitted when the scale locked state of the map changes
+     * \param locked true if the scale is locked
+     * \see setScaleLocked
+     * \since QGIS 3.18
+     */
+    void scaleLockChanged( bool locked );
+
+
     //! Emitted when the extents of the map change
     void extentsChanged();
 
@@ -1059,7 +1140,7 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     void themeChanged( const QString &theme );
 
     //! emit a message (usually to be displayed in a message bar)
-    void messageEmitted( const QString &title, const QString &message, Qgis::MessageLevel = Qgis::Info );
+    void messageEmitted( const QString &title, const QString &message, Qgis::MessageLevel = Qgis::MessageLevel::Info );
 
     /**
      * Emitted whenever an error is encountered during a map render operation.
@@ -1126,12 +1207,14 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     void resizeEvent( QResizeEvent *e ) override;
     void paintEvent( QPaintEvent *e ) override;
     void dragEnterEvent( QDragEnterEvent *e ) override;
+    bool viewportEvent( QEvent *event ) override;
 
     //! called when panning is in action, reset indicates end of panning
     void moveCanvasContents( bool reset = false );
 
     void dropEvent( QDropEvent *event ) override;
 
+    void showEvent( QShowEvent *event ) override;
 
     /// implementation struct
     class CanvasProperties;
@@ -1244,7 +1327,29 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     bool mJobCanceled = false;
 
     //! Labeling results from the recently rendered map
-    QgsLabelingResults *mLabelingResults = nullptr;
+    std::unique_ptr< QgsLabelingResults > mLabelingResults;
+
+    //! TRUE if the labeling results stored in mLabelingResults are outdated (e.g. as a result of an ongoing canvas render)
+    bool mLabelingResultsOutdated = false;
+
+    /**
+     * Rendered results from the recently rendered map.
+     * \since QGIS 3.22
+     */
+    std::unique_ptr< QgsRenderedItemResults > mRenderedItemResults;
+
+    /**
+     * Rendered results stored from previously rendered maps
+     * \since QGIS 3.22
+     */
+    std::unique_ptr< QgsRenderedItemResults > mPreviousRenderedItemResults;
+
+    /**
+     * TRUE if the rendered item results stored in mRenderedItemResults are outdated (e.g. as a result of an ongoing canvas render)
+     *
+     * \since QGIS 3.22
+     */
+    bool mRenderedItemResultsOutdated = false;
 
     //! Whether layers are rendered sequentially or in parallel
     bool mUseParallelRendering = false;
@@ -1306,6 +1411,8 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
 
     int mBlockItemPositionUpdates = 0;
 
+    QMetaObject::Connection mScreenDpiChangedConnection;
+
     /**
      * Returns the last cursor position on the canvas in geographical coordinates
      * \since QGIS 3.4
